/****************************************************************************
** Meta object code from reading C++ file 'fb2extractor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../kfilemetadata-6.26.0/src/extractors/fb2extractor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fb2extractor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13KFileMetaData12Fb2ExtractorE_t {};
} // unnamed namespace

template <> constexpr inline auto KFileMetaData::Fb2Extractor::qt_create_metaobjectdata<qt_meta_tag_ZN13KFileMetaData12Fb2ExtractorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KFileMetaData::Fb2Extractor"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Fb2Extractor, qt_meta_tag_ZN13KFileMetaData12Fb2ExtractorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KFileMetaData::Fb2Extractor::staticMetaObject = { {
    QMetaObject::SuperData::link<ExtractorPlugin::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KFileMetaData12Fb2ExtractorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KFileMetaData12Fb2ExtractorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13KFileMetaData12Fb2ExtractorE_t>.metaTypes,
    nullptr
} };

void KFileMetaData::Fb2Extractor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Fb2Extractor *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *KFileMetaData::Fb2Extractor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KFileMetaData::Fb2Extractor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13KFileMetaData12Fb2ExtractorE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "org.kde.kf5.kfilemetadata.ExtractorPlugin"))
        return static_cast< KFileMetaData::ExtractorPlugin*>(this);
    return ExtractorPlugin::qt_metacast(_clname);
}

int KFileMetaData::Fb2Extractor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ExtractorPlugin::qt_metacall(_c, _id, _a);
    return _id;
}
using namespace KFileMetaData;

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_Fb2Extractor[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x29,  'o',  'r',  'g',  '.',  'k', 
    'd',  'e',  '.',  'k',  'f',  '5',  '.',  'k', 
    'f',  'i',  'l',  'e',  'm',  'e',  't',  'a', 
    'd',  'a',  't',  'a',  '.',  'E',  'x',  't', 
    'r',  'a',  'c',  't',  'o',  'r',  'P',  'l', 
    'u',  'g',  'i',  'n', 
    // "className"
    0x03,  0x6c,  'F',  'b',  '2',  'E',  'x',  't', 
    'r',  'a',  'c',  't',  'o',  'r', 
    // "MetaData"
    0x04,  0xa3,  0x62,  'I',  'd',  0x75,  'o',  'r', 
    'g',  '.',  'k',  'd',  'e',  '.',  'e',  'p', 
    'u',  'b',  'e',  'x',  't',  'r',  'a',  'c', 
    't',  'o',  'r',  0x69,  'M',  'i',  'm',  'e', 
    'T',  'y',  'p',  'e',  's',  0xa2,  0x78,  0x1d, 
    'a',  'p',  'p',  'l',  'i',  'c',  'a',  't', 
    'i',  'o',  'n',  '/',  'x',  '-',  'f',  'i', 
    'c',  't',  'i',  'o',  'n',  'b',  'o',  'o', 
    'k',  '+',  'x',  'm',  'l',  0xa1,  0x67,  'V', 
    'e',  'r',  's',  'i',  'o',  'n',  0x63,  '0', 
    '.',  '0',  0x78,  0x20,  'a',  'p',  'p',  'l', 
    'i',  'c',  'a',  't',  'i',  'o',  'n',  '/', 
    'x',  '-',  'z',  'i',  'p',  '-',  'c',  'o', 
    'm',  'p',  'r',  'e',  's',  's',  'e',  'd', 
    '-',  'f',  'b',  '2',  0xa1,  0x67,  'V',  'e', 
    'r',  's',  'i',  'o',  'n',  0x63,  '0',  '.', 
    '0',  0x64,  'N',  'a',  'm',  'e',  0x6c,  'F', 
    'b',  '2',  'E',  'x',  't',  'r',  'a',  'c', 
    't',  'o',  'r', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(KFileMetaData::Fb2Extractor, Fb2Extractor, qt_pluginMetaDataV2_Fb2Extractor)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_Fb2Extractor[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x29,  'o',  'r',  'g',  '.',  'k', 
    'd',  'e',  '.',  'k',  'f',  '5',  '.',  'k', 
    'f',  'i',  'l',  'e',  'm',  'e',  't',  'a', 
    'd',  'a',  't',  'a',  '.',  'E',  'x',  't', 
    'r',  'a',  'c',  't',  'o',  'r',  'P',  'l', 
    'u',  'g',  'i',  'n', 
    // "className"
    0x03,  0x6c,  'F',  'b',  '2',  'E',  'x',  't', 
    'r',  'a',  'c',  't',  'o',  'r', 
    // "MetaData"
    0x04,  0xa3,  0x62,  'I',  'd',  0x75,  'o',  'r', 
    'g',  '.',  'k',  'd',  'e',  '.',  'e',  'p', 
    'u',  'b',  'e',  'x',  't',  'r',  'a',  'c', 
    't',  'o',  'r',  0x69,  'M',  'i',  'm',  'e', 
    'T',  'y',  'p',  'e',  's',  0xa2,  0x78,  0x1d, 
    'a',  'p',  'p',  'l',  'i',  'c',  'a',  't', 
    'i',  'o',  'n',  '/',  'x',  '-',  'f',  'i', 
    'c',  't',  'i',  'o',  'n',  'b',  'o',  'o', 
    'k',  '+',  'x',  'm',  'l',  0xa1,  0x67,  'V', 
    'e',  'r',  's',  'i',  'o',  'n',  0x63,  '0', 
    '.',  '0',  0x78,  0x20,  'a',  'p',  'p',  'l', 
    'i',  'c',  'a',  't',  'i',  'o',  'n',  '/', 
    'x',  '-',  'z',  'i',  'p',  '-',  'c',  'o', 
    'm',  'p',  'r',  'e',  's',  's',  'e',  'd', 
    '-',  'f',  'b',  '2',  0xa1,  0x67,  'V',  'e', 
    'r',  's',  'i',  'o',  'n',  0x63,  '0',  '.', 
    '0',  0x64,  'N',  'a',  'm',  'e',  0x6c,  'F', 
    'b',  '2',  'E',  'x',  't',  'r',  'a',  'c', 
    't',  'o',  'r', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(KFileMetaData::Fb2Extractor, Fb2Extractor)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
